# Generated by Django 2.2.3 on 2019-07-02 18:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('refs', '0002_auto_20190702_1258'),
    ]

    operations = [
        migrations.AddField(
            model_name='type_param',
            name='pick_list',
            field=models.TextField(blank=True, null=True),
        ),
    ]
