# Generated by Django 3.0.3 on 2020-02-09 18:30

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0044_auto_20200209_1741'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='progresses',
            name='guid',
        ),
    ]
