# Generated by Django 2.2.4 on 2019-08-21 01:00

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0005_auto_20190821_0056'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='Calendar_shits_days',
            new_name='Calendar_shifts_days',
        ),
    ]
