# Generated by Django 2.2.4 on 2019-08-23 19:31

from django.db import migrations
import isc_common.fields.code_field
import isc_common.fields.description_field
import isc_common.fields.name_field


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0041_shift_day_relax'),
    ]

    operations = [
        migrations.AddField(
            model_name='shift_day_relax',
            name='code',
            field=isc_common.fields.code_field.CodeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='shift_day_relax',
            name='description',
            field=isc_common.fields.description_field.DescriptionField(),
        ),
        migrations.AddField(
            model_name='shift_day_relax',
            name='name',
            field=isc_common.fields.name_field.NameField(blank=True, null=True),
        ),
    ]
