# Generated by Django 3.0.3 on 2020-02-09 17:28

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0042_progresses_type'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='progresses',
            name='showCancel',
        ),
        migrations.AddField(
            model_name='progresses',
            name='props',
            field=bitfield.models.BitField((('showCancel', 'Показать кнопку отменить процесс'),), db_index=True, default=0),
        ),
    ]
