# Generated by Django 3.0.7 on 2020-06-30 13:10

from django.db import migrations
import isc_common.fields.code_field


class Migration(migrations.Migration):

    dependencies = [
        ('reports', '0009_auto_20200612_2242'),
    ]

    operations = [
        migrations.AddField(
            model_name='jasper_reports',
            name='report_id',
            field=isc_common.fields.code_field.CodeStrictField(null=True),
        ),
    ]
