# Generated by Django 3.0.3 on 2020-02-09 15:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0039_progresses_guid'),
    ]

    operations = [
        migrations.AlterField(
            model_name='progresses',
            name='guid',
            field=models.UUIDField(),
        ),
    ]
