# Generated by Django 2.2.5 on 2019-09-07 04:43

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0060_auto_20190907_0438'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='messages_files_refs',
            unique_together={('message', 'messages_file')},
        ),
    ]
