# Generated by Django 2.2.12 on 2020-04-28 18:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('files_blk', '0002_auto_20200428_1751'),
    ]

    operations = [
        migrations.AddField(
            model_name='files',
            name='file_size',
            field=models.PositiveIntegerField(default=None),
        ),
    ]
