# Generated by Django 2.2.5 on 2019-09-09 10:42

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('twits', '0013_chat_usersview'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='chat_usersview',
            options={'managed': False, 'verbose_name': 'Участники чатов'},
        ),
    ]
