# Generated by Django 2.2.4 on 2019-08-25 10:06

from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0054_shift_day_relax_color'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='calendar_shifts_days',
            name='saturday',
        ),
        migrations.AddField(
            model_name='shifts',
            name='order',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='calendar_shifts_days',
            name='friday',
            field=isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, related_name='saturday_rel', to='clndr.Shift_day'),
        ),
    ]
