# Generated by Django 2.2.8 on 2019-12-04 11:33

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0070_auto_20190829_1017'),
    ]

    operations = [
        migrations.AddField(
            model_name='calendars',
            name='props',
            field=bitfield.models.BitField((('default', 'Текущий календарь (по-умолчанию)'),), db_index=True, default=0),
        ),
    ]
