# Generated by Django 2.2.1 on 2019-05-13 20:12

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0017_auto_20190421_1501'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Messages_view',
        ),
    ]
