# Generated by Django 3.0.3 on 2020-02-05 12:37

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0077_messages_theme_with_out_user_view'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Messages_theme_with_out_user_view',
        ),
    ]
