# Generated by Django 2.2.4 on 2019-08-24 19:44

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0052_auto_20190824_1940'),
    ]

    operations = [
        migrations.AlterField(
            model_name='holidays',
            name='calendar',
            field=isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='clndr.Calendars'),
        ),
    ]
