# Generated by Django 2.1.7 on 2019-02-16 23:23

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0006_auto_20190216_2317'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='user_permission',
            unique_together={('user', 'widget')},
        ),
        migrations.AlterUniqueTogether(
            name='usergroup_permission',
            unique_together={('usergroup', 'widget')},
        ),
    ]
