# Generated by Django 2.1.7 on 2019-03-06 13:34

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.code_field
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0008_auto_20190221_0934'),
    ]

    operations = [
        migrations.CreateModel(
            name='Params',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', isc_common.fields.code_field.CodeField()),
                ('value', models.TextField(blank=True, null=True)),
                ('user', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Сохраненные папаметры',
            },
        ),
        migrations.AlterUniqueTogether(
            name='params',
            unique_together={('user', 'key')},
        ),
    ]
